from shapely.geometry import shape, box

BBox = list[float]


def check_tweet_location_inside_bbox(tweet, bbox: BBox, accuracy: int = 0):
    polygon = None

    if accuracy == 0:
        return True

    if tweet.coordinates:
        polygon = shape(tweet._json["coordinates"])
    elif tweet.place:
        polygon = shape(tweet._json["place"]["bounding_box"])
    else:
        return False

    container = box(*bbox, ccw=True)

    contains = container.contains(polygon)
    overlaps = container.overlaps(polygon)

    if accuracy == 1:
        return contains or overlaps

    return contains



import os
import sys
import json
import argparse
import logging

import requests
import tweepy
from .geo import check_tweet_location_inside_bbox

logging.basicConfig(level=logging.INFO)
logger = logging.getLogger(__name__)


def get_location(q):
    if not q:
        return

    response = requests.get(
        "https://nominatim.openstreetmap.org/search",
        params={
            "q": q,
            "format": "json",
        },
    )
    data = response.json()

    if len(data) > 0:
        return data[0]

    logger.warning(f"Couldn't find location for: {q}")


def main(args):
    keys = {
        key: os.getenv(key)
        for key in (
            "TWITTER_API_KEY",
            "TWITTER_API_SECRET",
            "TWITTER_TOKEN_KEY",
            "TWITTER_TOKEN_SECRET",
        )
    }

    if missing := [key for key, value in keys.items() if not value]:
        sys.exit(f"Missing env variables: {missing}")

    stream = tweepy.Stream(*keys.values())
    filters = {}

    if location := get_location(args.location):
        slat, nlat, wlon, elon = (float(coord) for coord in location["boundingbox"])
        print("+ Location: ", location["display_name"], file=sys.stderr)
        print(
            "+ Map:",
            f"https://openstreetmap.org/relation/{location['osm_id']} / http://bboxfinder.com/#{slat},{wlon},{nlat},{elon}",
            file=sys.stderr,
        )
        filters["locations"] = (wlon, slat, elon, nlat)

    if args.query:
        filters["track"] = args.query.split(",")

    if not filters:
        sys.exit("No filters")

    print("+ Filters: ", filters, file=sys.stderr)

    def on_status(status):
        if "locations" in filters and not check_tweet_location_inside_bbox(
            tweet=status, bbox=filters["locations"], accuracy=args.x
        ):
            return
        if args.json:
            json.dump(status._json, sys.stdout)
        else:
            print(f"@{status.user.screen_name}: {status.text}")
            if args.verbose:
                print(f"// place: {status.coordinates or status.place.name}")
                print(f"// source: https://twitter.com/user/status/{status.id_str}")
            print()
            sys.stdout.flush()

    try:
        stream.on_status = on_status
        stream.filter(**filters)
    except KeyboardInterrupt:
        pass
    finally:
        stream.disconnect()


def cli():
    parser = argparse.ArgumentParser()
    parser.add_argument("-q", "--query", help="filter keywords separated by comma")
    parser.add_argument("-l", "--location", help="location string, geocoded by openstreemap's nominatim")
    parser.add_argument("-j", "--json", action="store_true", default=False, help="output json")
    parser.add_argument(
        "-x",
        action="count",
        help="location accuracy, -x (within/overlaps location bbox), -xx (within location bbox), defaults to simply intersects",
    )
    parser.add_argument("-v", "--verbose", action="store_true")
    args = parser.parse_args()
    main(args)


if __name__ == "__main__":
    cli()



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['twts']

package_data = \
{'': ['*']}

install_requires = \
['Shapely>=1.8.2,<2.0.0', 'requests>=2.28.0,<3.0.0', 'tweepy>=4.10.0,<5.0.0']

entry_points = \
{'console_scripts': ['twts = twts.stream:cli']}

setup_kwargs = {
    'name': 'twts',
    'version': '0.1.1',
    'description': 'stream tweets from the terminal',
    'long_description': None,
    'author': 'redraw',
    'author_email': 'redraw@sdf.org',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
